/*
 * Copyright 2010 Martin Schreiber
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef CPRIMITIVES_HPP
#define CPRIMITIVES_HPP

#include <GL/gl.h>
#include <cmath>

class CPrimitives
{
public:
	static void circle(int fans);

	static void round_plate(int fans, float scale = 1.0f);

	static void barrel(int parts, float scale_height = 1.0f, float scale_radius = 1.0f);

	static void plane(int lod = 10, float size=1.0f);

	static void sphere(float r, float cx,float cy,float cz, unsigned int LODi, float texScale = 1);

	static inline void originCross()
	{
		glColor3f(1.0, 0.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(-0.2, 0, 0);
			glVertex3f(1.0, 0, 0);
		glEnd();

		glColor3f(0.0, 1.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(0, -0.2,  0);
			glVertex3f(0, 1.0,  0);
		glEnd();


		glColor3f(0.0, 0.0, 1.0);
		glBegin(GL_LINES);
			glVertex3f(0, 0, -0.2);
			glVertex3f(0, 0, 1.0);
		glEnd();
	};

	/**
	 * very simple arrow
	 */
	static inline void arrow()
	{
		glColor3f(1.0, 0.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(-0.2, 0, 0);
			glVertex3f(1.0, 0, 0);
		glEnd();

		glColor3f(0.0, 1.0, 0.0);
		glBegin(GL_LINES);
			glVertex3f(0, -0.2,  0);
			glVertex3f(0, 0.2,  0);
		glEnd();


		glColor3f(0.0, 0.0, 1.0);
		glBegin(GL_LINES);
			glVertex3f(0, 0, -0.2);
			glVertex3f(0, 0, 0.2);
		glEnd();
	};


	static inline void box()
	{
		glBegin(GL_LINE_STRIP);
			glVertex3f(-0.5, -0.5, -0.5);
			glVertex3f(0.5, -0.5, -0.5);
			glVertex3f(0.5, 0.5, -0.5);
			glVertex3f(-0.5, 0.5, -0.5);
			glVertex3f(-0.5, -0.5, -0.5);
			glVertex3f(-0.5, -0.5, 0.5);
			glVertex3f(0.5, -0.5, 0.5);
			glVertex3f(0.5, 0.5, 0.5);
			glVertex3f(-0.5, 0.5, 0.5);
			glVertex3f(-0.5, -0.5, 0.5);
			glEnd();
		glBegin(GL_LINES);
			glVertex3f(0.5, -0.5, -0.5);
			glVertex3f(0.5, -0.5, 0.5);
			glVertex3f(0.5, 0.5, -0.5);
			glVertex3f(0.5, 0.5, 0.5);
			glVertex3f(-0.5, 0.5, -0.5);
			glVertex3f(-0.5, 0.5, 0.5);
		glEnd();
	}
};

#endif
